"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var universal_1 = require("../../universal");
exports.PromiseTracker = universal_1.PromiseTracker;
var errors_1 = require("./errors");
var recursive_task_resolver_1 = require("./helpers/recursive-task-resolver");
var render_app_to_string_1 = require("./render-app-to-string");
var router_context_handler_1 = require("./router-context-handler");
var ServerRenderResults = tslib_1.__importStar(require("./server-render-results"));
function renderErrorRoute(ssrRequestProps, options, currentRenderLocation, promiseTracker, err, startTime) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var errorRenderOptions, errorRender;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    errorRenderOptions = err instanceof errors_1.Status404Error
                        ? { location: options.pageNotFoundLocation, status: 404 }
                        : { location: options.errorLocation, status: 500 };
                    // If we are already rendering the error location, bail
                    // This will be caught above and a 500 will be returned
                    if (currentRenderLocation === errorRenderOptions.location) {
                        throw err;
                    }
                    // Reset the request state, don't want it for the error route
                    options.resetRequest(errorRenderOptions.location);
                    // Overwrite the render location with the error location
                    currentRenderLocation = errorRenderOptions.location;
                    return [4 /*yield*/, renderWithErrorPageFallback(ssrRequestProps, options, errorRenderOptions.location, promiseTracker, startTime)];
                case 1:
                    errorRender = _a.sent();
                    if (errorRender.type === ServerRenderResults.ServerRenderResultType.Success) {
                        // We have successfully rendered the error page, but it still needs
                        // to be an error status code (404 or 500)
                        errorRender.statusCode = errorRenderOptions.status;
                    }
                    return [2 /*return*/, errorRender];
            }
        });
    });
}
function performRender(ssrRequestProps, options, renderLocation, promiseTracker, startTime) {
    // We need to capture the store before we render
    // as any changes caused by the render will not be
    // included in the rendered content
    var storeStateAtRenderTime = tslib_1.__assign({}, ssrRequestProps);
    try {
        var renderResult = render_app_to_string_1.renderApp(renderLocation, options.renderFn, options.log, options.appRender, promiseTracker);
        var result = router_context_handler_1.routerContextHandler(renderResult, startTime, storeStateAtRenderTime, renderLocation);
        return result;
    }
    finally {
        if (options.events && options.events.renderPerformed) {
            try {
                options.events.renderPerformed(promiseTracker);
            }
            catch (err) {
                // External event failed. Just log and continue
                options.log.error({ err: err }, 'renderPerformed event failed');
            }
        }
    }
}
function renderWithErrorPageFallback(ssrRequestProps, options, renderLocation, promiseTracker, startTime) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var initialRenderResult, dataResolved, dataLoadErr_1;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    _a.trys.push([0, 2, , 4]);
                    initialRenderResult = performRender(ssrRequestProps, options, renderLocation, promiseTracker, startTime);
                    // If we have not rendered successfully just return the render result
                    if (initialRenderResult.type !== ServerRenderResults.ServerRenderResultType.Success) {
                        return [2 /*return*/, initialRenderResult];
                    }
                    if (options.events && options.events.beginWaitingForTasks) {
                        try {
                            options.events.beginWaitingForTasks(universal_1.formatElapsed(process.hrtime(startTime)));
                        }
                        catch (err) {
                            // external event failed, log and continue
                            options.log.error({ err: err, location: location }, 'beginWaitingForTasks threw, continuing');
                        }
                    }
                    return [4 /*yield*/, recursive_task_resolver_1.recursiveTaskResolver(options.log, promiseTracker, function () {
                            return performRender(ssrRequestProps, options, renderLocation, promiseTracker, startTime);
                        }, renderLocation, initialRenderResult, 10, options.ssrTimeoutMs)];
                case 1:
                    dataResolved = _a.sent();
                    return [2 /*return*/, dataResolved];
                case 2:
                    dataLoadErr_1 = _a.sent();
                    if (dataLoadErr_1 instanceof errors_1.Information) {
                        options.log.info({ err: dataLoadErr_1 }, 'Data load warning, rendering error location');
                    }
                    else if (dataLoadErr_1 instanceof errors_1.Warning) {
                        options.log.warn({ err: dataLoadErr_1 }, 'Data load warning, rendering error location');
                    }
                    else {
                        options.log.error({ err: dataLoadErr_1 }, 'Data load failed, rendering error location');
                    }
                    return [4 /*yield*/, renderErrorRoute(ssrRequestProps, options, renderLocation, promiseTracker, dataLoadErr_1, startTime)];
                case 3: return [2 /*return*/, _a.sent()];
                case 4: return [2 /*return*/];
            }
        });
    });
}
function renderPageContents(ssrRequestProps, options, location, promiseTracker) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var startTime, err_1, failure;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    startTime = process.hrtime();
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, renderWithErrorPageFallback(ssrRequestProps, options, location, promiseTracker, startTime)];
                case 2: return [2 /*return*/, _a.sent()];
                case 3:
                    err_1 = _a.sent();
                    failure = {
                        elapsed: universal_1.formatElapsed(process.hrtime(startTime)),
                        errorMessage: 'Failed to do render',
                        head: undefined,
                        type: ServerRenderResults.ServerRenderResultType.Failure,
                    };
                    options.log.error({ err: err_1 }, 'Failed to render');
                    return [2 /*return*/, failure];
                case 4: return [2 /*return*/];
            }
        });
    });
}
exports.renderPageContents = renderPageContents;
//# sourceMappingURL=full-render.js.map