import { __awaiter } from "tslib";
import path from 'path';
import SpeedMeasurePlugin from 'speed-measure-webpack-plugin';
import { ENVIRONMENTS, getWebpackConfig, TARGETS } from '../build/build';
import clean from '../clean';
import { watchtowerConfigFilename } from '../runtime/config/config';
import { writeFile, existsSync } from '../runtime/util/fs';
import { webpackPromise } from '../util/webpack';
import { getMd5, setupWithBuildInfo, validateCache } from './cache-validator';
import { getBabelConfigFile, getTsConfigFile } from '../config/ts-loader-config';
export function smp(buildConfig, webpackConfig) {
    const smpPlugin = new SpeedMeasurePlugin();
    return buildConfig.SMP ? smpPlugin.wrap(webpackConfig) : webpackConfig;
}
const buildTarget = (log, buildConfig, target, environment = 'prod') => __awaiter(void 0, void 0, void 0, function* () {
    const config = getWebpackConfig(log, buildConfig, target, environment);
    if (!config) {
        return Promise.reject(`Could not load webpack configuration for ${target}/${environment}!`);
    }
    const disableCaching = String(process.env.BUILD_CACHE_DISABLED).toLowerCase() === 'true';
    if (disableCaching) {
        log.info(`Build Caching Disabled... BUILD_CACHE_DISABLED=${disableCaching}`);
    }
    else if (process.env.NODE_ENV !== 'test' && !disableCaching) {
        const webpackConfigString = JSON.stringify(config);
        const configHash = yield getMd5(log, 'webpackConfig', webpackConfigString);
        const validationItems = [
            {
                filePath: getTsConfigFile(target, buildConfig),
                hashKey: 'tsconfigHash',
                isFile: true,
            },
            { isFile: false, itemHash: configHash, hashKey: 'webpackConfig' },
        ];
        const yarnLockPath = path.resolve(process.cwd(), 'yarn.lock');
        if (existsSync(yarnLockPath)) {
            validationItems.push({
                filePath: yarnLockPath,
                hashKey: 'yarnLockConfigHash',
                isFile: true,
            });
        }
        const npmLockPath = path.resolve(process.cwd(), 'package-lock.json');
        if (existsSync(npmLockPath)) {
            validationItems.push({
                filePath: npmLockPath,
                hashKey: 'npmLockConfigHash',
                isFile: true,
            });
        }
        const babelConfig = getBabelConfigFile(target, buildConfig);
        if (babelConfig && existsSync(babelConfig)) {
            validationItems.push({
                filePath: babelConfig,
                hashKey: 'babelConfigHash',
                isFile: true,
            });
        }
        setupWithBuildInfo(log, {
            buildInfo: {
                environment,
                project: buildConfig.BASE,
                target,
            },
            traceMessages: false,
            validationItems,
        });
        yield validateCache(log);
    }
    return webpackPromise(log, smp(buildConfig, config)).then(() => {
        const runtimeConfig = {
            ASSETS_PATH: buildConfig.ASSETS_PATH_PREFIX,
            ASSETS_PATH_PREFIX: buildConfig.ASSETS_PATH_PREFIX,
            BASE: '.',
            PUBLIC_PATH: buildConfig.PUBLIC_PATH,
            SERVER_PUBLIC_DIR: buildConfig.SERVER_PUBLIC_DIR === false ? false : 'public/',
        };
        // On success write out watchtower config
        return writeFile(log, path.join(buildConfig.OUTPUT, watchtowerConfigFilename), JSON.stringify(runtimeConfig, undefined, 2));
    });
});
const getBuildEnvironment = (args) => {
    for (const arg of args) {
        if (typeof arg !== 'string') {
            continue;
        }
        if (ENVIRONMENTS.indexOf(arg) !== -1) {
            return arg;
        }
    }
    return 'prod';
};
const getBuildTargets = (buildConfig, args) => {
    for (const arg of args) {
        if (!buildConfig.HAS_SERVER && arg === 'server') {
            continue;
        }
        if (typeof arg !== 'string') {
            continue;
        }
        if (TARGETS.indexOf(arg) !== -1) {
            return [arg];
        }
    }
    const defaultTargets = buildConfig.HAS_SERVER ? ['server', 'client'] : ['client'];
    return defaultTargets;
};
/**
 * Builds one or all targets in a specified environment
 * @param args
 *  [<target>] [<environment>] [-p <project root>]
 *  - target defaults to both server and client
 *  - environment defaults to prod
 *  - p: the root of the build, used for config discovery
 */
const build = (log, buildConfig, ...args) => __awaiter(void 0, void 0, void 0, function* () {
    const targets = getBuildTargets(buildConfig, args);
    const environment = getBuildEnvironment(args);
    const cleanTarget = buildConfig.OUTPUT;
    yield clean(log, cleanTarget);
    for (const target of targets) {
        yield buildTarget(log, buildConfig, target, environment);
    }
});
export default build;
//# sourceMappingURL=build.js.map