import { __awaiter } from "tslib";
import { renderToString } from 'react-dom/server';
import URL from 'url';
import { getRuntimeConfig } from '../../config/config';
import { PromiseTracker } from '../../util/promise-tracker';
import { getAssetLocations, getBodyAssets, getHeadAssets } from '../assets';
import { hasLog } from '../middleware/ensure-request-log-middleware';
import { renderPageContents, ServerRenderResultType, transferState, } from './';
export const createSsrMiddleware = (options) => {
    const runtimeConfig = getRuntimeConfig(options.app.log);
    const buildAssets = getAssetLocations(runtimeConfig);
    return (req, response, next) => __awaiter(void 0, void 0, void 0, function* () {
        if (!hasLog(req)) {
            // eslint-disable-next-line no-console
            console.error('Skipping SSR middleware due to missing req.log key');
            return next();
        }
        const promiseTracker = new PromiseTracker();
        let appState = yield options.setupRequest(req, promiseTracker);
        let renderContext;
        const ssrOptions = {
            appRender: () => {
                renderContext = {
                    promiseTracker,
                    ssrRequestProps: appState,
                };
                return options.renderApp({
                    context: renderContext,
                    log: req.log,
                    req,
                });
            },
            errorLocation: options.errorLocation,
            events: {
                // eslint-disable-next-line @typescript-eslint/no-empty-function
                renderPerformed: () => { },
            },
            log: req.log,
            pageNotFoundLocation: options.pageNotFoundLocation,
            // Not perfect typings, if someone specifies the render result type
            // as something it isn't, that's their own fault
            renderFn: options.renderFn || renderToString,
            resetRequest: (location) => __awaiter(void 0, void 0, void 0, function* () {
                req.url = location;
                req.query = URL.parse(req.url, true).query;
                appState = yield options.setupRequest(req, promiseTracker);
            }),
            ssrTimeoutMs: options.ssrTimeoutMs,
        };
        const pageRenderResult = yield renderPageContents(appState, ssrOptions, req.url, promiseTracker);
        const createPageMarkup = (result) => {
            const helmetTags = [];
            if (result.head && result.head.title) {
                helmetTags.push(result.head.title.toString());
            }
            if (result.head && result.head.base) {
                helmetTags.push(result.head.base.toString());
            }
            if (result.head && result.head.script) {
                helmetTags.push(result.head.script.toString());
            }
            if (result.head && result.head.meta) {
                helmetTags.push(result.head.meta.toString());
            }
            if (result.head && result.head.link) {
                helmetTags.push(result.head.link.toString());
            }
            const stateTransfers = [];
            // When watchtower renders an error, the client needs to know
            // so it can hydrate the error location. The WatchtowerBrowserRouter takes
            // care of this
            if (result.renderLocation !== req.originalUrl) {
                stateTransfers.push({
                    tag: transferState('watchtower_hydrate_location', result.renderLocation),
                });
            }
            const pageTags = options.createPageTags
                ? options.createPageTags({ buildAssets, helmetTags, stateTransfers, renderContext })
                : {
                    body: [...getBodyAssets(buildAssets)],
                    preBody: [],
                    head: [
                        ...helmetTags.map(tag => ({ tag })),
                        ...getHeadAssets(buildAssets),
                        ...stateTransfers,
                    ],
                };
            return options.renderHtml({
                context: renderContext,
                head: result.head,
                pageTags,
                renderResult: result.renderedContent,
                req,
            });
        };
        switch (pageRenderResult.type) {
            case ServerRenderResultType.Success:
                response
                    .status(pageRenderResult.statusCode)
                    .header('Content-Type', 'text/html')
                    .send(createPageMarkup(pageRenderResult));
                return;
            case ServerRenderResultType.Redirect:
                response.redirect(pageRenderResult.isPermanent ? 301 : 302, pageRenderResult.redirectTo);
                return;
            case ServerRenderResultType.Failure:
            default:
                // tslint:disable-line:no-switch-case-fall-through
                response.status(500).send();
        }
    });
};
//# sourceMappingURL=create-middleware.js.map