import { __awaiter } from "tslib";
const delay = (forMs) => new Promise(resolve => setTimeout(resolve, forMs));
const component = 'RecursiveTaskResolver';
const timedOut = Symbol('Timed out');
function innerResolve(log, promiseTracker, render, location, initialRender, numberAttempts, remainingAttempts, timeoutPromise) {
    if (!promiseTracker.hasWork()) {
        log.debug({ component, location }, 'No work, resolving');
        return Promise.resolve(initialRender);
    }
    if (!remainingAttempts || remainingAttempts <= 0) {
        return Promise.reject(new Error(`Abort waiting for loading all data after ${numberAttempts} recursive waits`));
    }
    return new Promise((resolve, reject) => setTimeout(() => {
        const promiseOrTimedOut = [promiseTracker.waitForCompletion(), timeoutPromise];
        Promise.race(promiseOrTimedOut)
            .then(p => {
            // If we have timed out return the timed out symbol to
            // prevent another nested render
            if (p === timedOut) {
                return p;
            }
            log.debug({ component, location }, 'Re-rendering to trigger any child promises');
            const renderResult = render();
            // eslint-disable-next-line consistent-return
            return innerResolve(log, promiseTracker, render, location, renderResult, numberAttempts, remainingAttempts - 1, timeoutPromise);
        })
            .then(resolve)
            .catch(reject);
    }));
}
/**
 * Utility which will recursively render the React application, triggering any
 * additional data loader recursively until no more data loads are triggered
 * or it times out, or it hits the max depth
 */
export function recursiveTaskResolver(log, promiseTracker, render, location, initialRender, numberAttempts, timeoutMs) {
    return __awaiter(this, void 0, void 0, function* () {
        const timeoutPromise = delay(timeoutMs).then(() => timedOut);
        const resolved = yield innerResolve(log, promiseTracker, render, location, initialRender, numberAttempts, numberAttempts, timeoutPromise);
        if (resolved === timedOut) {
            // Fall back to just rendering again
            log.error({ location }, `Resolving tasks timed out after ${timeoutMs}ms`);
            return render();
        }
        return resolved;
    });
}
//# sourceMappingURL=recursive-task-resolver.js.map