import { __awaiter } from "tslib";
import { fork } from 'child_process';
import dotenv from 'dotenv';
import express from 'express';
import proxyMiddleware from 'http-proxy-middleware';
import path from 'path';
import webpack from 'webpack';
import { getWebpackConfig } from '../build/build';
import { getPort } from '../runtime/server/server';
import { findFreePort, waitForConnection } from '../runtime/util/network';
import { getHotReloadMiddleware, openBrowser } from '../server/dev';
import { webpackStatsConfig } from '../util/webpack';
const restartServer = (buildConfig, port, projectDir, oldServer) => {
    if (oldServer) {
        oldServer.kill();
    }
    // When running in local dev, we have a different process.cwd() than
    // when running in production. This allows static files and such to resolve
    return fork(path.resolve(buildConfig.OUTPUT, 'server.js'), [], {
        env: Object.assign(Object.assign({}, process.env), { LOAD_DEFAULT_ASSETS: 'true', PORT: port.toString(), PROJECT_DIR: projectDir }),
    });
};
function watchServer(log, buildConfig) {
    return __awaiter(this, void 0, void 0, function* () {
        dotenv.config({
            path: path.join(buildConfig.BASE, '.env'),
        });
        const hostPort = yield findFreePort(getPort(buildConfig.DEV_SERVER_PORT));
        const devServerPort = yield findFreePort(hostPort + 1);
        let devServer;
        let devServerAvailable;
        const serverConfig = getWebpackConfig(log, buildConfig, 'server', 'dev');
        const serverCompiler = webpack(serverConfig);
        serverCompiler.hooks.invalid.tap('invalid', () => {
            log.info('⭐  Server changed, rebuilding and restarting server...  ⭐');
        });
        const watching = serverCompiler.watch({}, (err, stats) => {
            if (!devServer) {
                setTimeout(() => openBrowser(log, hostPort), 2000);
            }
            if (err) {
                log.error({ err }, 'Failed to compile');
                return;
            }
            else {
                const statsString = stats.toString(webpackStatsConfig);
                log.info(statsString);
                if (stats.hasErrors()) {
                    log.error('Stats has errors');
                    return;
                }
            }
            devServer = restartServer(buildConfig, devServerPort, buildConfig.BASE, devServer);
            setTimeout(() => {
                devServerAvailable = waitForConnection(devServerPort);
            }, 100);
        });
        const app = express();
        app.use(getHotReloadMiddleware(log, buildConfig));
        app.use((_req, _res, next) => __awaiter(this, void 0, void 0, function* () {
            yield devServerAvailable;
            next();
        }));
        app.use(proxyMiddleware('http://localhost:' + devServerPort));
        const watchServer = yield new Promise(resolve => {
            const server = app.listen(hostPort, () => {
                resolve({
                    app,
                    close: () => {
                        return Promise.all([
                            new Promise(closeResolve => watching.close(() => {
                                closeResolve();
                            })),
                            new Promise(closeResolve => server.close(() => closeResolve())),
                        ]).then(() => {
                            if (devServer) {
                                devServer.kill();
                            }
                        });
                    },
                    server,
                });
            });
        });
        app.set('server', watchServer);
        return watchServer;
    });
}
export default watchServer;
//# sourceMappingURL=server.js.map